@@ -1,5 +1,5 @@
 /*
- * (c) Copyright IBM Corp 2001, 2005 
+ * (c) Copyright IBM Corp 2001, 2006 
  */
 
 package com.ibm.wsdl.xml;
@@ -15,6 +15,7 @@ import javax.wsdl.*;
 import javax.wsdl.extensions.*;
 import javax.wsdl.factory.*;
 import javax.wsdl.xml.*;
+
 import com.ibm.wsdl.*;
 import com.ibm.wsdl.util.*;
 import com.ibm.wsdl.util.xml.*;
@@ -119,6 +120,7 @@ public class WSDLReaderImpl implements WSDLReader
    * Gets the value of the specified feature.
    *
    * @param name the name of the feature to get the value of.
+   * @return the value of the feature.
    * @throws IllegalArgumentException if the feature name is not recognized.
    * @see #setFeature(String, boolean)
    */
@@ -202,11 +204,17 @@ public class WSDLReaderImpl implements WSDLReader
   public void setFactoryImplName(String factoryImplName)
     throws UnsupportedOperationException
   {
-    //the factory object is cached in the reader so set it
-    //to null if the factory impl name is reset.
-    this.factory = null;
-      
-    this.factoryImplName = factoryImplName;
+    //check to see if we really need to change the factory name and clear the cache
+    if((this.factoryImplName == null && factoryImplName != null) ||
+       (this.factoryImplName != null && !this.factoryImplName.equals(factoryImplName))) 
+    {
+      //the factory object is cached in the reader so set it
+      //to null if the factory impl name is reset.
+      this.factory = null;
+        
+      this.factoryImplName = factoryImplName;
+      //if(verbose) System.out.println("WSDLFactory Impl Name set to : "+factoryImplName);
+    }
   }
 
   /**
@@ -237,9 +245,15 @@ public class WSDLReaderImpl implements WSDLReader
                                              Constants.ATTR_TARGET_NAMESPACE);
     NamedNodeMap attrs = defEl.getAttributes();
 
+    if (importedDefs == null)
+    {
+      importedDefs = new Hashtable();
+    }
+    
     if (documentBaseURI != null)
     {
       def.setDocumentBaseURI(documentBaseURI);
+      importedDefs.put(documentBaseURI, def);
     }
 
     if (name != null)
@@ -280,16 +294,6 @@ public class WSDLReaderImpl implements WSDLReader
     {
       if (QNameUtils.matches(Constants.Q_ELEM_IMPORT, tempEl))
       {
-        if (importedDefs == null)
-        {
-          importedDefs = new Hashtable();
-        }
-
-        if (documentBaseURI != null)
-        {
-          importedDefs.put(documentBaseURI, def);
-        }
-
         def.addImport(parseImport(tempEl, def, importedDefs));
       }
       else if (QNameUtils.matches(Constants.Q_ELEM_DOCUMENTATION, tempEl))
@@ -325,6 +329,8 @@ public class WSDLReaderImpl implements WSDLReader
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
     }
 
+    parseExtensibilityAttributes(defEl, Definition.class, def, def);
+    
     return def;
   }
 
@@ -500,14 +506,18 @@ public class WSDLReaderImpl implements WSDLReader
           {
            throw e;
           }
-          catch (Throwable t)
+          catch (RuntimeException e)
+          {
+            throw e;
+          }
+          catch (Exception e)
           {
             throw new WSDLException(WSDLException.OTHER_ERROR,
                                     "Unable to resolve imported document at '" +
                                     locationURI + 
                                     (contextURI == null 
                                     ? "'." : "', relative to '" + contextURI + "'")
-                                    , t);
+                                    , e);
           }
         } //end importDocs
       } //end locationURI
@@ -531,6 +541,10 @@ public class WSDLReaderImpl implements WSDLReader
 	  throw e; 
 	}
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = importEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(importEl);
 
     while (tempEl != null)
@@ -541,7 +555,8 @@ public class WSDLReaderImpl implements WSDLReader
       }
       else
       {
-        DOMUtils.throwWSDLException(tempEl);
+        importDef.addExtensibilityElement(
+          parseExtensibilityElement(Import.class, tempEl, def));        
       }
 
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
@@ -556,14 +571,10 @@ public class WSDLReaderImpl implements WSDLReader
   protected Types parseTypes(Element typesEl, Definition def)
     throws WSDLException
   {
-    List remainingAttrs = DOMUtils.getAttributes(typesEl);
-    
-    //This element cannot contain extension attributes, so check for
-    //unexpected remaining attributes.
-    if (!remainingAttrs.isEmpty()) {
-        DOMUtils.throwWSDLException(typesEl, remainingAttrs);;
-    }
-      
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = typesEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Types types = def.createTypes();
     Element tempEl = DOMUtils.getFirstChildElement(typesEl);
     QName tempElType;
@@ -591,6 +602,8 @@ public class WSDLReaderImpl implements WSDLReader
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
     }
 
+    parseExtensibilityAttributes(typesEl, Types.class, types, def);
+    
     return types;
   }
   
@@ -840,15 +853,19 @@ public class WSDLReaderImpl implements WSDLReader
   	    {
   	      throw e;
   	    }
-  	    catch (Throwable t)
+            catch (RuntimeException e)
+            {
+              throw e;
+            }
+  	    catch (Exception e)
   	    {
-  	      throw new WSDLException(WSDLException.OTHER_ERROR,
+              throw new WSDLException(WSDLException.OTHER_ERROR,
   	                "An error occurred trying to resolve schema referenced at '" 
   	  	            + schemaRef.getSchemaLocationURI() 
   	  	            + "'"
   		            + (schema.getDocumentBaseURI() == null ? "." : ", relative to '"
   		            + schema.getDocumentBaseURI() + "'."),
-  	  	            t);
+  	  	            e);
   	    }
   	    
   	  } //end while loop
@@ -889,13 +906,7 @@ public class WSDLReaderImpl implements WSDLReader
                                                     Constants.ELEM_BINDING,
                                                     def,
                                                     remainingAttrs);
-    
-    //This element cannot contain extension attributes, so check for
-    //unexpected remaining attributes.
-    if (!remainingAttrs.isEmpty()) {
-        DOMUtils.throwWSDLException(bindingEl, remainingAttrs);;
-    }
-    
+        
     PortType portType = null;
 
     if (name != null)
@@ -932,6 +943,10 @@ public class WSDLReaderImpl implements WSDLReader
       binding.setPortType(portType);
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = bindingEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(bindingEl);
 
     while (tempEl != null)
@@ -971,17 +986,15 @@ public class WSDLReaderImpl implements WSDLReader
                                         Constants.ATTR_NAME,
                                         remainingAttrs);
     
-    //This element cannot contain extension attributes, so check for
-    //unexpected remaining attributes.
-    if (!remainingAttrs.isEmpty()) {
-        DOMUtils.throwWSDLException(bindingOperationEl, remainingAttrs);;
-    }
-
     if (name != null)
     {
       bindingOperation.setName(name);
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = bindingOperationEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(bindingOperationEl);
 
     while (tempEl != null)
@@ -1015,11 +1028,45 @@ public class WSDLReaderImpl implements WSDLReader
     {
       BindingInput bindingInput = bindingOperation.getBindingInput();
       BindingOutput bindingOutput = bindingOperation.getBindingOutput();
-      String inputName =
-        (bindingInput != null ? bindingInput.getName() : null);
-      String outputName =
-        (bindingOutput != null ? bindingOutput.getName() : null);
+      String inputName = (bindingInput != null 
+              ? (bindingInput.getName() != null ? bindingInput.getName() : Constants.NONE)
+              : null);
+      String outputName = (bindingOutput != null 
+              ? (bindingOutput.getName() != null ? bindingOutput.getName() : Constants.NONE) 
+              : null);
       Operation op = portType.getOperation(name, inputName, outputName);
+      
+      /*
+       * If the bindingOp input or output message names are null we will search first
+       * for a porttypeOp with corresponding unnamed input or output messages (using  
+       * Constants.NONE for inputName or outputName, as above). 
+       * However, input and output message names need not be used at all if operation 
+       * overloading is not used, so if no match was found we will try again ignoring 
+       * these unnamed messages from the search criteria (i.e. using null instead of 
+       * Constants.NONE for inputName or outputName).
+       */
+      
+      if(op == null)
+      {
+        if(Constants.NONE.equals(inputName) && Constants.NONE.equals(outputName))
+        {
+          //There was no porttype op with unnamed input and output messages,
+          //so ignore input and output name and search on the op name only.
+          op = portType.getOperation(name, null, null);
+        }
+        else if(Constants.NONE.equals(inputName))
+        {
+          //There was no porttype op with an unnamed input message,
+          //so ignore input name and search on the op name and output name only.
+          op = portType.getOperation(name, null, outputName);
+        }
+        else if(Constants.NONE.equals(outputName))
+        {
+          //There was no porttype op with an unnamed output message,
+          //so ignore output name and search on the op name and input name only.
+          op = portType.getOperation(name, inputName, null);
+        }
+      }
 
       if (op == null)
       {
@@ -1038,6 +1085,8 @@ public class WSDLReaderImpl implements WSDLReader
       bindingOperation.setOperation(op);
     }
 
+    parseExtensibilityAttributes(bindingOperationEl, BindingOperation.class, bindingOperation, def);
+    
     return bindingOperation;
   }
 
@@ -1052,17 +1101,15 @@ public class WSDLReaderImpl implements WSDLReader
                                         Constants.ATTR_NAME,
                                         remainingAttrs);
     
-    //This element cannot contain extension attributes, so check for
-    //unexpected remaining attributes.
-    if (!remainingAttrs.isEmpty()) {
-        DOMUtils.throwWSDLException(bindingInputEl, remainingAttrs);;
-    }
-
     if (name != null)
     {
       bindingInput.setName(name);
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = bindingInputEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(bindingInputEl);
 
     while (tempEl != null)
@@ -1094,17 +1141,15 @@ public class WSDLReaderImpl implements WSDLReader
                                         Constants.ATTR_NAME,
                                         remainingAttrs);
     
-    //This element cannot contain extension attributes, so check for
-    //unexpected remaining attributes.
-    if (!remainingAttrs.isEmpty()) {
-        DOMUtils.throwWSDLException(bindingOutputEl, remainingAttrs);;
-    }
-
     if (name != null)
     {
       bindingOutput.setName(name);
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = bindingOutputEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(bindingOutputEl);
 
     while (tempEl != null)
@@ -1135,18 +1180,16 @@ public class WSDLReaderImpl implements WSDLReader
     String name = DOMUtils.getAttribute(bindingFaultEl,
                                         Constants.ATTR_NAME,
                                         remainingAttrs);
-
-    //This element cannot contain extension attributes, so check for
-    //unexpected remaining attributes.
-    if (!remainingAttrs.isEmpty()) {
-        DOMUtils.throwWSDLException(bindingFaultEl, remainingAttrs);;
-    }
     
     if (name != null)
     {
       bindingFault.setName(name);
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = bindingFaultEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(bindingFaultEl);
 
     while (tempEl != null)
@@ -1164,6 +1207,8 @@ public class WSDLReaderImpl implements WSDLReader
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
     }
 
+    parseExtensibilityAttributes(bindingFaultEl, BindingFault.class, bindingFault, def);
+    
     return bindingFault;
   }
 
@@ -1174,12 +1219,6 @@ public class WSDLReaderImpl implements WSDLReader
     
     List remainingAttrs = DOMUtils.getAttributes(msgEl);
     String name = DOMUtils.getAttribute(msgEl, Constants.ATTR_NAME, remainingAttrs);
-
-    //This element cannot contain extension attributes, so check for
-    //unexpected remaining attributes.
-    if (!remainingAttrs.isEmpty()) {
-        DOMUtils.throwWSDLException(msgEl, remainingAttrs);;
-    }
     
     if (name != null)
     {
@@ -1201,6 +1240,10 @@ public class WSDLReaderImpl implements WSDLReader
     // Whether it was retrieved or created, the definition has been found.
     msg.setUndefined(false);
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = msgEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(msgEl);
 
     while (tempEl != null)
@@ -1222,6 +1265,8 @@ public class WSDLReaderImpl implements WSDLReader
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
     }
 
+    parseExtensibilityAttributes(msgEl, Message.class, msg, def);
+    
     return msg;
   }
 
@@ -1254,6 +1299,10 @@ public class WSDLReaderImpl implements WSDLReader
       part.setTypeName(typeName);
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = partEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(partEl);
 
     while (tempEl != null)
@@ -1264,7 +1313,8 @@ public class WSDLReaderImpl implements WSDLReader
       }
       else
       {
-        DOMUtils.throwWSDLException(tempEl);
+        part.addExtensibilityElement(
+          parseExtensibilityElement(Part.class, tempEl, def));
       }
 
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
@@ -1407,6 +1457,10 @@ public class WSDLReaderImpl implements WSDLReader
     // Whether it was retrieved or created, the definition has been found.
     portType.setUndefined(false);
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = portTypeEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(portTypeEl);
 
     while (tempEl != null)
@@ -1426,7 +1480,8 @@ public class WSDLReaderImpl implements WSDLReader
       }
       else
       {
-        DOMUtils.throwWSDLException(tempEl);
+        portType.addExtensibilityElement(
+          parseExtensibilityElement(PortType.class, tempEl, def));
       }
 
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
@@ -1449,13 +1504,11 @@ public class WSDLReaderImpl implements WSDLReader
     String parameterOrderStr = DOMUtils.getAttribute(opEl,
                                                      Constants.ATTR_PARAMETER_ORDER,
                                                      remainingAttrs);
-    
-    //This element cannot contain extension attributes, so check for
-    //unexpected remaining attributes.
-    if (!remainingAttrs.isEmpty()) {
-        DOMUtils.throwWSDLException(opEl, remainingAttrs);;
-    }
-    
+        
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = opEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(opEl);
     List messageOrder = new Vector();
     Element docEl = null;
@@ -1496,8 +1549,12 @@ public class WSDLReaderImpl implements WSDLReader
 
     if (name != null)
     {
-      String inputName = (input != null ? input.getName() : null);
-      String outputName = (output != null ? output.getName() : null);
+      String inputName = (input != null 
+              ? (input.getName() != null ? input.getName() : Constants.NONE) 
+              : null);
+      String outputName = (output != null 
+              ? (output.getName() != null ? output.getName() : Constants.NONE) 
+              : null);
 
       op = portType.getOperation(name, inputName, outputName);
 
@@ -1624,6 +1681,8 @@ public class WSDLReaderImpl implements WSDLReader
       op = null;
     }
 
+    parseExtensibilityAttributes(opEl, Operation.class, op, def);
+    
     return op;
   }
 
@@ -1635,17 +1694,15 @@ public class WSDLReaderImpl implements WSDLReader
     List remainingAttrs = DOMUtils.getAttributes(serviceEl);
     String name = DOMUtils.getAttribute(serviceEl, Constants.ATTR_NAME, remainingAttrs);
     
-    //This element cannot contain extension attributes, so check for
-    //unexpected remaining attributes.
-    if (!remainingAttrs.isEmpty()) {
-        DOMUtils.throwWSDLException(serviceEl, remainingAttrs);;
-    }
-
     if (name != null)
     {
       service.setQName(new QName(def.getTargetNamespace(), name));
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = serviceEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(serviceEl);
 
     while (tempEl != null)
@@ -1667,6 +1724,8 @@ public class WSDLReaderImpl implements WSDLReader
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
     }
 
+    parseExtensibilityAttributes(serviceEl, Service.class, service, def);
+    
     return service;
   }
 
@@ -1683,12 +1742,6 @@ public class WSDLReaderImpl implements WSDLReader
                                                   def,
                                                   remainingAttrs);
     
-    //This element cannot contain extension attributes, so check for
-    //unexpected remaining attributes.
-    if (!remainingAttrs.isEmpty()) {
-        DOMUtils.throwWSDLException(portEl, remainingAttrs);;
-    }
-
     if (name != null)
     {
       port.setName(name);
@@ -1708,6 +1761,10 @@ public class WSDLReaderImpl implements WSDLReader
       port.setBinding(binding);
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = portEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(portEl);
 
     while (tempEl != null)
@@ -1726,6 +1783,8 @@ public class WSDLReaderImpl implements WSDLReader
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
     }
 
+    parseExtensibilityAttributes(portEl, Port.class, port, def);
+    
     return port;
   }
 
@@ -1809,6 +1868,10 @@ public class WSDLReaderImpl implements WSDLReader
       input.setMessage(message);
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = inputEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(inputEl);
 
     while (tempEl != null)
@@ -1819,7 +1882,8 @@ public class WSDLReaderImpl implements WSDLReader
       }
       else
       {
-        DOMUtils.throwWSDLException(tempEl);
+        input.addExtensibilityElement(
+          parseExtensibilityElement(Input.class, tempEl, def));
       }
 
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
@@ -1859,6 +1923,10 @@ public class WSDLReaderImpl implements WSDLReader
       output.setMessage(message);
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = outputEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(outputEl);
 
     while (tempEl != null)
@@ -1869,7 +1937,8 @@ public class WSDLReaderImpl implements WSDLReader
       }
       else
       {
-        DOMUtils.throwWSDLException(tempEl);
+        output.addExtensibilityElement(
+          parseExtensibilityElement(Output.class, tempEl, def));
       }
 
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
@@ -1909,6 +1978,10 @@ public class WSDLReaderImpl implements WSDLReader
       fault.setMessage(message);
     }
 
+    //register any NS decls with the Definition
+    NamedNodeMap attrs = faultEl.getAttributes();
+    registerNSDeclarations(attrs, def);
+
     Element tempEl = DOMUtils.getFirstChildElement(faultEl);
 
     while (tempEl != null)
@@ -1919,7 +1992,8 @@ public class WSDLReaderImpl implements WSDLReader
       }
       else
       {
-        DOMUtils.throwWSDLException(tempEl);
+        fault.addExtensibilityElement(
+            parseExtensibilityElement(Fault.class, tempEl, def));
       }
 
       tempEl = DOMUtils.getNextSiblingElement(tempEl);
@@ -2028,14 +2102,43 @@ public class WSDLReaderImpl implements WSDLReader
 
       return doc;
     }
-    catch (Throwable t)
+    catch (RuntimeException e)
+    {
+      throw e;
+    }
+    catch (Exception e)
     {
       throw new WSDLException(WSDLException.PARSER_ERROR,
-                              "Problem parsing '" + desc + "'.",
-                              t);
+                                "Problem parsing '" + desc + "'.",
+                                e);
     }
   }
 
+  private static void registerNSDeclarations(NamedNodeMap attrs, Definition def)
+  {
+      int size = attrs.getLength();
+
+      for (int i = 0; i < size; i++)
+      {
+        Attr attr = (Attr)attrs.item(i);
+        String namespaceURI = attr.getNamespaceURI();
+        String localPart = attr.getLocalName();
+        String value = attr.getValue();
+
+        if (namespaceURI != null && namespaceURI.equals(Constants.NS_URI_XMLNS))
+        {
+          if (localPart != null && !localPart.equals(Constants.ATTR_XMLNS))
+          {
+            DOMUtils.registerUniquePrefix(localPart, value, def);
+          }
+          else
+          {
+            DOMUtils.registerUniquePrefix(null, value, def);
+          }
+        }
+      }
+  }
+  
   /**
    * Read the WSDL document accessible via the specified
    * URI into a WSDL definition.
@@ -2092,7 +2195,11 @@ public class WSDLReaderImpl implements WSDLReader
     {
       throw e;
     }
-    catch (Throwable t)
+    catch (RuntimeException e)
+    {
+      throw e;
+    }
+    catch (Exception e)
     {
       throw new WSDLException(WSDLException.OTHER_ERROR,
                               "Unable to resolve imported document at '" +
@@ -2100,7 +2207,7 @@ public class WSDLReaderImpl implements WSDLReader
                               (contextURI == null
                               ? "'."
                               : "', relative to '" + contextURI + "'.")
-                              , t);
+                              , e);
     }
   }
 
@@ -2122,6 +2229,34 @@ public class WSDLReaderImpl implements WSDLReader
     return readWSDL(documentBaseURI, definitionsElement, null);
   }
 
+  /**
+   * Read the specified &lt;wsdl:definitions&gt; element into a WSDL
+   * definition. The WSDLLocator is used to provide the document
+   * base URIs. The InputSource of the WSDLLocator is ignored, instead
+   * the WSDL is parsed from the given Element. 
+   *
+   * @param locator A WSDLLocator object used to provide 
+   * the document base URI of the WSDL definition described by the
+   * element.
+   * @param definitionsElement the &lt;wsdl:definitions&gt; element
+   * @return the definition described by the element.
+   */
+  public Definition readWSDL(WSDLLocator locator,
+                             Element definitionsElement)
+                               throws WSDLException
+  {
+    try
+    {
+      this.loc = locator;
+      return readWSDL(locator.getBaseURI(), definitionsElement, null);
+    }
+    finally
+    {
+      locator.close();
+      this.loc = null;
+    }
+  }
+  
   protected Definition readWSDL(String documentBaseURI,
                                 Element definitionsElement,
                                 Map importedDefs)
@@ -2195,6 +2330,14 @@ public class WSDLReaderImpl implements WSDLReader
       System.out.println("Retrieving document at '" + base + "'.");
     }
 
-    return readWSDL(base, is);
+    try
+    {
+      return readWSDL(base, is);
+    }
+    finally
+    {
+      this.loc.close();
+      this.loc = null;
+    }
   }
 }
